﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Clients.Hive.Views.Administration {
  [View("ResourcesView")]
  [Content(typeof(IItemList<Resource>), IsDefaultView = true)]
  public partial class ResourcesView : ItemView {
    public new IItemList<Resource> Content {
      get { return (IItemList<Resource>)base.Content; }
      set { base.Content = value; }
    }

    public ResourcesView() {
      InitializeComponent();
      treeSlaveGroup.ImageList.Images.Add(HiveImageLibrary.Slave);
      treeSlaveGroup.ImageList.Images.Add(HiveImageLibrary.SlaveGroup);
      updateScheduleControl.UpdateAction = new Action(UpdateSchedule);
    }

    private void UpdateSchedule() {
      Guid resourceId = scheduleView.ResourceId;
      if (resourceId != null) {
        ServiceLocator.Instance.CallHiveService(service => {
          var appointments = service.GetScheduleForResource(resourceId);
          ItemList<Appointment> ias = new ItemList<Appointment>();
          appointments.ForEach(a => ias.Add(a));
          scheduleView.Invoke(new Action(() => scheduleView.Content = ias));
        });
      }
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers on the Content here
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers on the Content here
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        slaveView.Content = null;
        treeSlaveGroup.Nodes.Clear();
      } else {

        TreeNode ungrp = new TreeNode("UNGROUPED");
        ungrp.ImageIndex = treeSlaveGroup.ImageList.Images.Count - 1;
        ungrp.SelectedImageIndex = ungrp.ImageIndex;
        var newGroup = new SlaveGroup();
        newGroup.Name = "UNGROUPED";
        newGroup.Id = Guid.NewGuid();
        newGroup.Description = "Contains slaves which are in no group";
        ungrp.Tag = newGroup;

        foreach (Resource g in Content) {
          if (g.GetType() == typeof(SlaveGroup)) {
            TreeNode tn = new TreeNode();
            tn.ImageIndex = treeSlaveGroup.ImageList.Images.Count - 1;
            tn.SelectedImageIndex = tn.ImageIndex;

            tn.Tag = g;
            tn.Text = g.Name;
            foreach (Slave s in Content.Where(s => s.ParentResourceId != null && s.ParentResourceId == g.Id)) {
              var stn = new TreeNode(s.Name);
              stn.ImageIndex = 0;
              stn.SelectedImageIndex = stn.ImageIndex;
              stn.Tag = s;
              tn.Nodes.Add(stn);
            }
            treeSlaveGroup.Nodes.Add(tn);


          } else if (g.GetType() == typeof(Slave)) {
            if (g.ParentResourceId == null) {
              var stn = new TreeNode(g.Name);
              stn.ImageIndex = 0;
              stn.SelectedImageIndex = stn.ImageIndex;
              stn.Tag = g;
              ungrp.Nodes.Add(stn);
            }
          }
        }
        treeSlaveGroup.Nodes.Add(ungrp);
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: Put code here to enable or disable controls based on whether the Content is/not null or the view is ReadOnly
    }

    private void treeSlaveGroup_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e) {
      if (e.Node.Tag.GetType() == typeof(Slave)) {
        slaveView.Content = (Slave)e.Node.Tag;
        scheduleView.ResourceId = ((Slave)e.Node.Tag).Id;
      }
    }

    #region Event Handlers
    // TODO: Put event handlers here
    #endregion
  }
}
