﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Clients.Hive.Views {
  [View("ItemTree View")]
  [Content(typeof(IItemTree), IsDefaultView = false)]
  public sealed partial class ItemTreeView : ItemView {
    public new IItemTree Content {
      get { return (IItemTree)base.Content; }
      set { base.Content = value; }
    }

    public ItemTreeView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      // Deregister your event handlers here
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // Register your event handlers here
    }

    #region Event Handlers (Content)
    // Put event handlers of the content here
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // Add code when content has been changed and is null
        treeView.Nodes.Clear();
      } else {
        // Add code when content has been changed and is not null
        treeView.Nodes.Clear();
        AddChildNodes(Content, treeView.Nodes);
      }
    }

    private void AddChildNodes(IItemTree item, TreeNodeCollection nodes) {
      var node = CreateTreeViewNode(item);
      nodes.Add(node);
      var childItems = item.GetChildNodes();
      foreach (var childItem in childItems) {        
        AddChildNodes(childItem, node.Nodes);
      }
    }

    private TreeNode CreateTreeViewNode(IItemTree item) {
      var node = new TreeNode(item.ToString());
      node.Tag = item;
      if (!treeView.ImageList.Images.ContainsKey(item.ItemImage.GetHashCode().ToString())) {
        treeView.ImageList.Images.Add(item.ItemImage.GetHashCode().ToString(), item.ItemImage);
      }
      node.ImageKey = item.ItemImage.GetHashCode().ToString();
      return node;
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // Enable or disable controls based on whether the content is null or the view is set readonly
    }

    #region Event Handlers (child controls)
    // Put event handlers of child controls here.
    private void treeView_AfterSelect(object sender, TreeViewEventArgs e) {
      viewHost.Content = (IContent)treeView.SelectedNode.Tag;
    }
    #endregion


  }
}
